import Foundation
import metaoneSDK
import SwiftUI


@objc(MetaoneWalletReactNativeSdk)
class MetaoneWalletReactNativeSdk: NSObject {
    private var metaOneSDKManager = MetaOneSDKManager()
    private var hostingController: UIHostingController<AnyView>? = nil
    private var isOpened: Bool = false
    
    override init() {
        super.init()
        self.metaOneSDKManager.events.subscribe { event in
        print("self.metaOneSDKManager.events.subscribe:::","\(event)")
          switch event {
            case .showSDKUI:
              if self.isOpened == false {
                  self.openSDKUI()
              }
            case .hideSDKUI:
              if self.isOpened == true {
                  self.closeSDKUI()
              }
            @unknown default:
                fatalError()
            }
       }
    }
    
    @objc(initialize:resolve:reject:)
    func initialize(sdkConfig:NSDictionary,  resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        if let config = sdkConfig as? [String: Any] {
            metaOneSDKManager.initialize(
                sdkConfig: SDKConfig(
                    SDK_VERSION: (config["version"] as? String)!,
                    SDK_REALM: (config["sdkRealm"] as? String)!,
                    SDK_ENVIRONMENT: (config["sdkEnvironment"] as? String)!,
                    SDK_KEY: (config["sdkKey"] as? String)!,
                    SDK_CONFIG_URL: (config["sdkConfigUrl"] as? String)!,
                    SDK_PUB_PHRASE: (config["sdkApiKeyPhrase"] as? String)!,
                    SDK_API_CLIENT_REFERENCE: (config["sdkApiClientReference"] as? String)!
                )
            )
            resolve(true)
        } else {
            reject("Invalid Config", "SDK Config is not valid", nil)
        }
    }
    
    @objc func getSessionActivityStatus(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let result = metaOneSDKManager.getSessionActivityStatus()
        resolve(result.rawValue)
    }
    
    @objc(loginWithSSO:resolve:reject:)
    func loginWithSSO(accessToken:String,  resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let loginRequest = AuthApiModel.SSOLoginRequest(token: accessToken)
        let callback = M1EnqueueCallback<Bool>(
            onSuccess: { _ in
                self.metaOneSDKManager.setup(completion: {
                    resolve(true)
                })
            },
            onError: { resp in
                if !APIError.isLinkRequired(code: resp.code, responseCode: resp.responseCode) {
                    resolve(false)
                }
            }
        )
        metaOneSDKManager.login(requestData: loginRequest, callback: callback)
    }
    
    @objc func getExpireAt(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let expiredAt = metaOneSDKManager.getExpireAt()
        resolve(expiredAt)
    }
    
    @objc func refreshSession(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        metaOneSDKManager.refreshSession(callback: PromiseCallback<Bool>(resolve: resolve, reject: reject).enqueueCallback)
    }
    
    @objc func isSignatureSet(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let setPin = metaOneSDKManager.isSignatureSet()
        resolve(setPin)
    }
    
    @objc(sendTransaction:address:amount:asset:memo:personalNote:resolve:reject:)
    func sendTransaction(wallet:NSDictionary, address:String, amount:String, asset:NSDictionary?, memo:String, personalNote:String,  resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        do {
            let walletJsonString = Helper().toString(dictionary: wallet)
            var selectedAsset: Wallets.WalletToken? = nil

            if let asset = asset, let assetJsonString = Helper().toString(dictionary: asset) {
                if let data = assetJsonString.data(using: .utf8) {
                    selectedAsset = try? JSONDecoder().decode(Wallets.WalletToken.self, from: data)
                }
            }
            
            let selectedWallet = try JSONDecoder().decode(Wallets.UserWallet.self, from: Data(walletJsonString!.utf8))
            metaOneSDKManager.sendTransaction(wallet: selectedWallet, address: address, amount: amount, asset: selectedAsset, memo: memo, personalNote: personalNote, callback: PromiseCallback<Bool>(resolve: resolve, reject: reject, onClose: {self.closeSDKUI()}).enqueueCallback)
        } catch {
            reject("ERROR_CODE", "SendTransaction: \(error)", nil)
        }
    }
    
    @objc func openWallet(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        if let result = metaOneSDKManager.openWallet(){
            reject("ERROR_CODE", "\(result)", nil)
        } else{
            resolve(true)
        }
    }
    
    
    @objc func getColorsScheme(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let colorsJsonString = metaOneSDKManager.uiManager.getCurrentColorScheme().toJsonString()
        resolve(Helper().toDictionary(text: colorsJsonString!))
    }
    
    @objc func setColorsScheme(_ colors: NSDictionary) {
        let jsonString = Helper().toString(dictionary: colors)
        let colorScheme = AppConfig.ColorsScheme.fromJsonString(jsonString: jsonString!)
        metaOneSDKManager.uiManager.setCurrentColorScheme(colorScheme: colorScheme)
    }
    
    
    @objc func getCurrentLanguage(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    {
        let currentLanguage = metaOneSDKManager.uiManager.getCurrentLanguage()
        
        let locale: String
        switch currentLanguage {
        case  SupportedLanguage.ENGLISH_US : locale = "en-US"
        case  SupportedLanguage.VIETNAMESE : locale = "vi-VN"
        case  SupportedLanguage.INDONESIAN : locale = "in-ID"
        case  SupportedLanguage.FILIPINO : locale = "tl-PH"
        case  SupportedLanguage.CHINESE_SIMPLIFIED : locale = "zh-CN"
        case  SupportedLanguage.CHINESE_TRADITIONAL : locale = "zh-TW"
        default: locale = "en-US"
        }
        
        resolve(locale)
    }
    
    @objc func setCurrentLanguage(_ locale: String) {
        let currentLanguage: SupportedLanguage
        switch locale {
        case "en-US": currentLanguage = SupportedLanguage.ENGLISH_US
        case "vi-VN": currentLanguage = SupportedLanguage.VIETNAMESE
        case "in-ID": currentLanguage = SupportedLanguage.INDONESIAN
        case "tl-PH": currentLanguage = SupportedLanguage.FILIPINO
        case "zh-CN": currentLanguage = SupportedLanguage.CHINESE_SIMPLIFIED
        case "zh-TW": currentLanguage = SupportedLanguage.CHINESE_TRADITIONAL
        default: currentLanguage = SupportedLanguage.ENGLISH_US
        }
        metaOneSDKManager.uiManager.setCurrentLanguage(locale: currentLanguage)
    }
    
    @objc func getWallets(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    
    {
        metaOneSDKManager.apiManager.getWallets(callback: PromiseCallback<UserWalletsResponse>(resolve: resolve, reject: reject).enqueueCallback)
    }
    
    @objc func getCurrencies(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock)
    
    {
        metaOneSDKManager.apiManager.getCurrencies(callback: PromiseCallback<UserCurrenciesResponse>(resolve: resolve, reject: reject).enqueueCallback)
    }
    
    @objc(getTransactions:assetRef:bip44:tokenAddress:limit:offset:resolve:reject:)
    func getTransactions(
        walletId:String?,
        assetRef:String?,
        bip44:String?,
        tokenAddress:String?,
        limit:String,
        offset:String,
        resolve:@escaping RCTPromiseResolveBlock,
        reject: @escaping RCTPromiseRejectBlock
    ){
        metaOneSDKManager.apiManager.getTransactions(
            walletId: walletId,
            assetRef: assetRef,
            bip44: bip44,
            tokenAddress: tokenAddress,
            page: Int(limit),
            offset: Int(offset),
            callback: PromiseCallback<TransactionAPIModel.TransactionsResponse>(resolve: resolve, reject: reject).enqueueCallback
        )
        
    }
    @objc(getNFTs:searchString:limit:offset:resolve:reject:)
    func getNFTs(
        walletId: String?, searchString: String?, limit: String, offset: String, resolve:@escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock
    ){
        metaOneSDKManager.apiManager.getNfts(
            walletId: walletId,
            queryString: searchString,
            limit: Int(limit)!,
            offset: Int(offset)!,
            callback: PromiseCallback<UserNFTsResponse>(resolve: resolve, reject: reject).enqueueCallback
        )
    }
    
    @objc func getUserContacts(_ resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    @objc(getUserContactWithId:resolve:reject:)
    func getUserContactWithId(contactId:String, resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    @objc(addUserContact:resolve:reject:)
    func addUserContact(contact:NSDictionary, resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    @objc(updateUserContact:contact:resolve:reject:)
    func updateUserContact(contactId: String, contact:NSDictionary, resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    
    @objc(deleteUserContact:resolve:reject:)
    func deleteUserContact(contactId: String, resolve: @escaping RCTPromiseResolveBlock, reject: @escaping RCTPromiseRejectBlock){
        
    }
    
    
    @objc func logout()
    {
        DispatchQueue.main.async {
            self.metaOneSDKManager.logout()
        }
    }
    
    
    func openSDKUI () {
        self.isOpened = true
        DispatchQueue.main.async {
            let controller = UIHostingController(rootView: AnyView(self.metaOneSDKManager.sdkView.environmentObject(self.metaOneSDKManager.uiManager)))
            if let rootViewController = UIApplication.shared.delegate?.window??.rootViewController {
                rootViewController.addChild(controller)
                rootViewController.view.addSubview(controller.view)
                controller.view.translatesAutoresizingMaskIntoConstraints = false
                NSLayoutConstraint.activate([
                    controller.view.topAnchor.constraint(equalTo: rootViewController.view.topAnchor),
                    controller.view.bottomAnchor.constraint(equalTo: rootViewController.view.bottomAnchor),
                    controller.view.leadingAnchor.constraint(equalTo: rootViewController.view.leadingAnchor),
                    controller.view.trailingAnchor.constraint(equalTo: rootViewController.view.trailingAnchor)
                ])
                controller.view.frame = CGRect(x: UIScreen.main.bounds.width, y: 0, width: controller.view.frame.width, height: controller.view.frame.height)
                UIView.animate(withDuration: 0.3, animations: {
                    controller.view.frame = CGRect(x: 0, y: 0, width: controller.view.frame.width, height: controller.view.frame.height)
                })
                controller.didMove(toParent: rootViewController)
                self.hostingController = controller
            }
        }
    }
    
    func closeSDKUI () {
        self.isOpened = false
        if let  controller = self.hostingController {
            DispatchQueue.main.async {
                UIView.animate(withDuration: 0.3, animations: {
                    controller.view.frame = CGRect(x: UIScreen.main.bounds.width, y: 0, width: controller.view.frame.width, height: controller.view.frame.height)
                }, completion: {finished in
                    controller.willMove(toParent: nil)
                    controller.view.removeFromSuperview()
                    controller.removeFromParent()
                    self.hostingController = nil
                })
            }
        }
    }
}
